package com.zfire.mall.manager.logic.admin;

import com.zfire.mall.manager.bean.admin.AdminModuleTree;
import com.zfire.mall.manager.bean.admin.AdminUserCom;
import com.zfire.mall.manager.exception.RemoteServiceException;
import com.zfire.mall.manager.logic.common.CommonLogic;
import com.zfire.mall.manager.plus.entity.AdminModule;
import com.zfire.mall.manager.plus.entity.AdminModuleFavorites;
import com.zfire.mall.manager.plus.service.AdminModuleFavoritesService;
import com.zfire.mall.manager.plus.service.AdminModuleService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;


@Service
public class AdminModuleLogic {

    @Autowired
    AdminModuleService adminModuleService;
    @Autowired
    AdminModuleFavoritesService adminModuleFavoritesService;
    @Autowired
    CommonLogic commonLogic;


    /**
     * 新增模块
     * @param adminModule
     */
    public void add(AdminModule adminModule){
        Integer level = 1;
        if(!adminModule.getParentId().equals("0")){
            AdminModule parent = adminModuleService.getById(adminModule.getParentId());
            if(parent == null){
                throw new RemoteServiceException("请选择正确的父级菜单");
            }
            level = parent.getLevel() + 1;
//            parent.setType(1);
//            parent.updateById();
        }
        Integer count = adminModuleService.lambdaQuery()
                .and(v -> v.eq(AdminModule::getFullUrl, adminModule.getFullUrl()).or().eq(AdminModule::getCode, adminModule.getCode()))
                .count();
        if(count > 0){
            throw new RemoteServiceException("code或者url重复");
        }
//        adminModule.setUrl(adminModule.getUrl().replaceAll(" ",""));
//        if(adminModule.getUrl().indexOf("http:") > -1 || adminModule.getUrl().indexOf("https:") > -1){
//            adminModule.setType(4);
//        }else{
//            adminModule.setType(2);
//        }
        adminModule.setLevel(level);
        adminModule.setCreateTime(new Date());
        adminModule.setCurUrl(adminModule.getFullUrl());
        adminModule.insert();
    }


    public void update(AdminModule adminModule){
        Integer level = 1;
        if(!adminModule.getParentId().equals("0")){
            AdminModule parent = adminModuleService.getById(adminModule.getParentId());
            if(parent == null){
                throw new RemoteServiceException("请选择正确的父级菜单");
            }
            level = parent.getLevel() + 1;
            //parent.setType(1);
            //parent.updateById();
        }
        Integer count = adminModuleService.lambdaQuery()
                .and(v -> v.eq(AdminModule::getFullUrl, adminModule.getFullUrl()).or().eq(AdminModule::getCode, adminModule.getCode()))
                .ne(AdminModule::getModuleId,adminModule.getModuleId())
                .count();
        if(count > 0){
            throw new RemoteServiceException("code或者url重复");
        }
        adminModule.setUrl(adminModule.getUrl().replaceAll(" ",""));
//        if(adminModule.getUrl().indexOf("http:") > -1 || adminModule.getUrl().indexOf("https:") > -1){
//            adminModule.setType(4);
//        }else{
//            //查询是否有子项
//            Integer count1 = adminModuleService.lambdaQuery()
//                    .eq(AdminModule::getParentId, adminModule.getModuleId())
//                    .ne(AdminModule::getType,3)
//                    .count();
//            if(count1 > 0){
//                adminModule.setType(1);
//            }else {
//                adminModule.setType(2);
//            }
//        }
        adminModule.setLevel(level);
        adminModule.updateById();
    }


    public void delete(String id){
        adminModuleService.removeById(id);
    }


    public AdminModule detail(String id){
        return adminModuleService.getById(id);
    }


    /**
     * 设置常用菜单
     */
    @Transactional
    public void setupFavorites(List<String> adminModuleIds){
        AdminUserCom adminUser = commonLogic.getAdminUser();
        if(adminModuleIds.size() > 10 || adminModuleIds.size() == 0){
            throw new RemoteServiceException("请选择1-10个常用菜单");
        }
        //1.remove all
        adminModuleFavoritesService.lambdaUpdate().eq(AdminModuleFavorites::getAdminUserId,adminUser.getAdminUserId()).remove();
        //2.add
        List<AdminModuleFavorites> list = new ArrayList<>();
        for(String adminModuleId : adminModuleIds){
            AdminModuleFavorites adminModuleFavorites = new AdminModuleFavorites();
            adminModuleFavorites.setAdminModuleId(adminModuleId);
            adminModuleFavorites.setAdminUserId(adminUser.getAdminUserId());
            list.add(adminModuleFavorites);
        }
        adminModuleFavoritesService.saveBatch(list);
    }

    /**
     * 常用菜单列表
     */
    public List<AdminModule> listFavorites(){
        AdminUserCom adminUser = commonLogic.getAdminUser();
        List<AdminModuleFavorites> list = adminModuleFavoritesService.lambdaQuery().eq(AdminModuleFavorites::getAdminUserId, adminUser.getAdminUserId()).list();
        if(list.size() == 0){
            return new ArrayList<>();
        }
        List<String> adminModuleIds = list.stream().map(AdminModuleFavorites::getAdminModuleId).collect(Collectors.toList());
        List<AdminModule> adminModules = adminModuleService.lambdaQuery().in(AdminModule::getModuleId, adminModuleIds).list();
        return adminModules;
    }


}
